import React, { useState, useEffect } from 'react'
import styled from 'styled-components'
import { Button, message, Modal, Space, Table, Form, Input, Switch } from 'antd'
import {
  updatePsd,
  createJptProgram,
  getJptProgramList,
  startJupyter,
  stopJupyter,
} from '../services'
import moment from 'moment'

const JptWrapper = styled.div`
  padding: 20px;
  .create_btn {
    display: flex;
    justify-content: end;
    margin-bottom: 20px;
  }
`

const FormItem = Form.Item
const { Password } = Input

const JptMgmt = () => {
  const [jptList, setJptList] = useState([])
  const [dataLoading, setDataLoading] = useState(false)
  // 开关加载状态
  const [switchLoading, setSwitchLoading] = useState(false)
  const [isCreateJptProgram, setIsCreateJptProgram] = useState(false)
  const [isEditJptPsd, setIsEditJptPsd] = useState(false)
  const [currentJpt, setCurrentJpt] = useState(null)
  const [jptForm] = Form.useForm()
  const [psdForm] = Form.useForm()

  useEffect(() => {
    fetchJptList()
  }, [localStorage.getItem('project_id')])

  // 切换状态
  const changeTaskState = (checked, id) => {
    checked ? startJpt(id) : stopJpt(id)
  }

  const fetchJptList = async () => {
    setDataLoading(true)
    const { data } = await getJptProgramList()
    if (data.code === 200) {
      const list = data.data.map(item => {
        return {
          key: item.id,
          name: item.name,
          createTime: moment(item.create_time * 1000).format(
            'YYYY.MM.DD HH:mm:ss'
          ),
          creator: item.user_name,
          programStatus: item.status === 1,
          url: item.complete_url,
        }
      })
      setJptList(list)
    } else {
      message.error(data.msg)
    }
    setDataLoading(false)
  }

  const startJpt = async programme_id => {
    setSwitchLoading(true)
    const { data } = await startJupyter({ programme_id })
    if (data.code === 200) {
      message.success('开启成功')
      fetchJptList()
    } else {
      message.error(data.msg)
    }
    setSwitchLoading(false)
  }
  const stopJpt = async programme_id => {
    setSwitchLoading(true)
    const { data } = await stopJupyter({ programme_id })
    if (data.code === 200) {
      message.success('关闭成功')
      fetchJptList()
    } else {
      message.error(data.msg)
    }
    setSwitchLoading(false)
  }

  const columns = [
    {
      title: '程序名称',
      dataIndex: 'name',
      key: 'name',
    },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      key: 'createTime',
    },
    {
      title: '创建人',
      dataIndex: 'creator',
      key: 'creator',
    },
    {
      title: '状态',
      dataIndex: 'programStatus',
      key: 'programStatus',
      render: (val, record) => (
        <Switch
          checkedChildren="开启"
          unCheckedChildren="关闭"
          loading={switchLoading}
          defaultChecked={val}
          disabled={!['1', '2', '4'].includes(localStorage.getItem('role'))}
          onChange={checked => changeTaskState(checked, record.key)}
        />
      ),
    },
    {
      title: '操作',
      key: 'operation',
      render: (_, record) => (
        <Space size="middle">
          <Button
            type="link"
            disabled={!record.programStatus}
            onClick={() => {
              window.open(record.url, '_target')
            }}>
            编码
          </Button>
          <Button
            type="link"
            disabled={!['1', '2', '4'].includes(localStorage.getItem('role'))}
            onClick={() => {
              setCurrentJpt(record.key)
              setIsEditJptPsd(true)
            }}>
            修改密码
          </Button>
        </Space>
      ),
    },
  ]
  const createJpt = async () => {
    jptForm.validateFields().then(async () => {
      const { name, password } = jptForm.getFieldValue()
      const { data } = await createJptProgram({ name, password })
      if (data.code === 200) {
        message.success('创建成功')
        jptForm.resetFields()
        setIsCreateJptProgram(false)
        fetchJptList()
      } else {
        message.error(data.msg)
      }
    })
  }

  const createModal = (
    <Modal
      title="创建程序"
      visible={isCreateJptProgram}
      onOk={createJpt}
      okText="创建"
      onCancel={() => {
        jptForm.resetFields()
        setIsCreateJptProgram(false)
      }}>
      <Form form={jptForm} labelCol={{ span: 6 }} wrapperCol={{ span: 16 }}>
        <FormItem
          name="name"
          label="程序名称"
          rules={[
            {
              required: true,
              message: '请输入程序名称!',
            },
          ]}>
          <Input placeholder="请输入程序名称" />
        </FormItem>
        <FormItem
          name="password"
          label="密码"
          rules={[
            {
              required: true,
              message: '请输入密码!',
            },
          ]}>
          <Password placeholder="请输入密码" />
        </FormItem>
      </Form>
    </Modal>
  )

  const editJptPsd = () => {
    psdForm.validateFields().then(async () => {
      const { repassword, password } = psdForm.getFieldValue()
      if (repassword === password) {
        const { data } = await updatePsd({ programme_id: currentJpt, password })
        if (data.code === 200) {
          message.success('修改成功')
          psdForm.resetFields()
          setIsEditJptPsd(false)
          fetchJptList()
        } else {
          message.error(data.msg)
        }
      } else {
        message.error('两次密码不一致')
      }
    })
  }

  const editPsdModal = (
    <Modal
      title="修改密码"
      visible={isEditJptPsd}
      onOk={editJptPsd}
      okText="修改"
      onCancel={() => {
        psdForm.resetFields()
        setIsEditJptPsd(false)
      }}>
      <Form form={psdForm} labelCol={{ span: 6 }} wrapperCol={{ span: 16 }}>
        <FormItem
          name="password"
          label="密码"
          rules={[
            {
              required: true,
              message: '请输入密码!',
            },
          ]}>
          <Password placeholder="请输入密码" />
        </FormItem>
        <FormItem
          name="repassword"
          label="确认新密码"
          rules={[
            {
              required: true,
              message: '请输入密码!',
            },
          ]}>
          <Password placeholder="请输入密码" />
        </FormItem>
      </Form>
    </Modal>
  )

  return (
    <JptWrapper>
      <div className="create_btn">
        {jptList.length === 0 &&
          ['1', '2', '4'].includes(localStorage.getItem('role')) && (
            <Button
              type="primary"
              onClick={() => {
                setIsCreateJptProgram(true)
              }}>
              创建
            </Button>
          )}
      </div>
      <Table
        columns={columns}
        dataSource={jptList}
        bordered
        loading={dataLoading}
      />
      {createModal}
      {editPsdModal}
    </JptWrapper>
  )
}

export default JptMgmt
